#include <SoftwareSerial.h> 
SoftwareSerial MIDI(255, 3); // Rx, Tx (Rx not used, Tx pin=3)

void setup() {
  MIDI.begin(31250);         // Set up serial comm to MIDI
  resetMidi(4);              // Reset MIDI device (pin 8)
  ctrlMIDI(0, 0x07, 127);    // Set channel volume (0-127) to full
  ctrlMIDI(0, 0x00, 0x79);   // Select 'melodic' bank
  sendMIDI(0xC0|0, 0, 0);    // Select instrument (acoustic grand) in bank
}

void loop() {
  noteOn(0, 60, 127);        // Turn on middle-C note, velocity=127
  delay(1000);
  noteOff(0, 60,  127);      // Turn off note
  delay(1000);
}

// Turn note on (press key)
void noteOn(byte channel, byte note, byte attackVelo) {
  sendMIDI( (0x90 | channel), note, attackVelo);
}

// Turn note off (release key)
void noteOff(byte channel, byte note, byte releaseVelo) {
  sendMIDI( (0x80 | channel), note, releaseVelo);
}

// Set controller value with channel
void ctrlMIDI(byte channel, byte data1, byte data2) {
  sendMIDI( (0xB0 | channel), data1, data2);
}

// Send bytes to MIDI device
void sendMIDI(byte cmd, byte data1, byte data2) {
  MIDI.write(cmd);
  MIDI.write(data1);
  // Pass only if valid 2nd byte
  if( (cmd & 0xF0) <= 0xB0 || (cmd & 0xF0) == 0xE0 )
    MIDI.write(data2);
}
  
// Reset MIDI device
void resetMidi(int resetPin) {
  pinMode(resetPin, OUTPUT);
  digitalWrite(resetPin, LOW);
  delay(100);
  digitalWrite(resetPin, HIGH);
  delay(100);
}
